@@ -14,7 +14,7 @@ from git_machete.client import MacheteClient
 from git_machete.constants import EscapeCodes
 from git_machete.docs import short_docs, long_docs
 from git_machete.exceptions import MacheteException, StopInteraction
-from git_machete.git_operations import GitContext
+from git_machete.git_operations import AnyRevision, GitContext, LocalBranchShortName, RemoteBranchShortName
 from git_machete.utils import fmt, underline, excluding, warn
 
 T = TypeVar('T')
@@ -30,17 +30,18 @@ alias_by_command: Dict[str, str] = {
 }
 
 command_groups: List[Tuple[str, List[str]]] = [
-    # 'is-managed' is mostly for scripting use and therefore skipped
     ("General topics",
-     ["file", "help", "hooks", "version"]),
+     ["file", "format", "help", "hooks", "version"]),
     ("Build, display and modify the tree of branch dependencies",
      ["add", "anno", "discover", "edit", "status"]),
     ("List, check out and delete branches",
-     ["delete-unmanaged", "go", "list", "show"]),
+     ["delete-unmanaged", "go", "is-managed", "list", "show"]),
     ("Determine changes specific to the given branch",
      ["diff", "fork-point", "log"]),
     ("Update git history in accordance with the tree of branch dependencies",
-     ["advance", "reapply", "slide-out", "squash", "traverse", "update"])
+     ["advance", "reapply", "slide-out", "squash", "traverse", "update"]),
+    ("Integrate with third party tools",
+     ["github"])
 ]
 
 
@@ -109,7 +110,7 @@ class MacheteHelpAction(argparse.Action):
         # parser name (prog) is expected to be `git machete` or `git machete <command>`
         command_name = parser.prog.replace('git machete', '').strip()
         print(get_help_description(command_name))
-        parser.exit()
+        parser.exit(status=None)
 
 
 def create_cli_parser() -> argparse.ArgumentParser:
@@ -357,15 +358,25 @@ def create_cli_parser() -> argparse.ArgumentParser:
 def update_cli_opts_using_parsed_args(
         cli_opts: git_machete.options.CommandLineOptions,
         parsed_args: argparse.Namespace) -> None:
+
+    # Warning: In mypy, Arguments that come from untyped functions/variables are silently treated by mypy as Any.
+    # Since argparse is not typed, everything that comes from argparse.Namespace will be taken as Any :(
+    # Even if we add type=LocalBranchShortName into argument parser for branch,
+    # python debugger will see branch as LocalBranchShortName but mypy always will see it as Any,
+    # until you specifically tell mypy what is the exact type by casting (right now it's done this way below, but casting does not solve all of the problems).
+    #
+    # The reasonable solution here would be to use Typed Argument Parser which is a wrapper over argparse with modernised solution for typing.
+    # But it would add external dependency to git-machete, so let's stick to current casting.
+
     for opt, arg in vars(parsed_args).items():
         if opt == "branch":
-            cli_opts.opt_branch = arg
+            cli_opts.opt_branch = LocalBranchShortName.of(arg)
         elif opt == "checked_out_since":
             cli_opts.opt_checked_out_since = arg
         elif opt == "color":
             cli_opts.opt_color = arg
         elif opt == "down_fork_point":
-            cli_opts.opt_down_fork_point = arg
+            cli_opts.opt_down_fork_point = AnyRevision.of(arg)
         elif opt == "debug":
             cli_opts.opt_debug = True
         elif opt == "draft":
@@ -373,7 +384,7 @@ def update_cli_opts_using_parsed_args(
         elif opt == "fetch":
             cli_opts.opt_fetch = True
         elif opt == "fork_point":
-            cli_opts.opt_fork_point = arg
+            cli_opts.opt_fork_point = AnyRevision.of(arg)
         elif opt == "inferred":
             cli_opts.opt_inferred = True
         elif opt == "list_commits_with_hashes":
@@ -396,7 +407,7 @@ def update_cli_opts_using_parsed_args(
         elif opt == "no_push_untracked":
             cli_opts.opt_push_untracked = False
         elif opt == "onto":
-            cli_opts.opt_onto = arg
+            cli_opts.opt_onto = LocalBranchShortName.of(arg)
         elif opt == "override_to":
             cli_opts.opt_override_to = arg
         elif opt == "override_to_inferred":
@@ -411,7 +422,7 @@ def update_cli_opts_using_parsed_args(
         elif opt == "as_root":
             cli_opts.opt_as_root = True
         elif opt == "roots":
-            cli_opts.opt_roots = arg.split(",")
+            cli_opts.opt_roots = list(map(LocalBranchShortName.of, arg.split(",")))
         elif opt == "return_to":
             cli_opts.opt_return_to = arg
         elif opt == "stat":
@@ -454,31 +465,31 @@ def set_utils_global_variables(
     utils.verbose_mode = cli_opts.opt_verbose
 
 
-def validate_python_version() -> None:
-    if sys.version_info[:2] < (3, 6):
-        version_str = f"{sys.version_info.major}.{sys.version_info.minor}.{sys.version_info.micro}"
-        sys.stderr.write(
-            f"Python {version_str} is no longer supported. "
-            "Please switch to Python 3.6 or higher.\n")
-        sys.exit(1)
-
-
 def get_branch_arg_or_current_branch(
-        cli_opts: git_machete.options.CommandLineOptions, git_context: GitContext) -> str:
+        cli_opts: git_machete.options.CommandLineOptions, git_context: GitContext) -> LocalBranchShortName:
     return cli_opts.opt_branch or git_context.get_current_branch()
 
 
+def exit_script(status_code: Optional[int] = None) -> None:
+    # Single point of exit is useful, because we can mock this method in tests
+    # and verify that actual errors like MacheteException are raised with
+    # appropriate messages. Otherwise it's not possible, because SystemError
+    # caused by sys.exit() is overshadowing actual errors. As of the issue #287,
+    # there are some additional points of exit from the script in client.py
+    # method MacheteClient.pick() there are left on purpose since removing them
+    # would enforce significant refactor.
+    sys.exit(status_code)
+
+
 def launch(orig_args: List[str]) -> None:
     if not orig_args:
         print(get_help_description())
-        sys.exit(2)
+        exit_script(2)
 
     cli_opts = git_machete.options.CommandLineOptions()
-    git = GitContext(cli_opts)
+    git = GitContext()
 
     try:
-        machete_client = MacheteClient(cli_opts, git)
-
         cli_parser: argparse.ArgumentParser = create_cli_parser()
         parsed_cli: argparse.Namespace = cli_parser.parse_args(orig_args)
         parsed_cli_as_dict: Dict[str, str] = vars(parsed_cli)
@@ -489,7 +500,13 @@ def launch(orig_args: List[str]) -> None:
 
         cmd = parsed_cli.command
 
-        if cmd not in {'help', 'discover'}:
+        if cmd == "help":
+            print(get_help_description(parsed_cli.topic_or_cmd))
+            exit_script()
+
+        machete_client = MacheteClient(git)
+
+        if cmd != 'discover':
             if not os.path.exists(machete_client.definition_file_path):
                 # We're opening in "append" and not "write" mode to avoid a race condition:
                 # if other process writes to the file between we check the
@@ -508,13 +525,17 @@ def launch(orig_args: List[str]) -> None:
         if cmd == "add":
             machete_client.read_definition_file()
             branch = get_branch_arg_or_current_branch(cli_opts, git)
-            machete_client.add(branch)
+            machete_client.add(
+                branch=branch,
+                opt_onto=cli_opts.opt_onto,
+                opt_as_root=cli_opts.opt_as_root,
+                opt_yes=cli_opts.opt_yes)
         elif cmd == "advance":
             machete_client.read_definition_file()
             git.expect_no_operation_in_progress()
             current_branch = git.get_current_branch()
             machete_client.expect_in_managed_branches(current_branch)
-            machete_client.advance(current_branch)
+            machete_client.advance(branch=current_branch, opt_yes=cli_opts.opt_yes)
         elif cmd == "anno":
             machete_client.read_definition_file(verify_branches=False)
             if cli_opts.opt_sync_github_prs:
@@ -528,13 +549,17 @@ def launch(orig_args: List[str]) -> None:
                     machete_client.print_annotation(branch)
         elif cmd == "delete-unmanaged":
             machete_client.read_definition_file()
-            machete_client.delete_unmanaged()
+            machete_client.delete_unmanaged(opt_yes=cli_opts.opt_yes)
         elif cmd in {"diff", alias_by_command["diff"]}:
             machete_client.read_definition_file()
-            machete_client.diff(parsed_cli.branch)  # passing None if not specified
+            machete_client.diff(branch=parsed_cli.branch, opt_stat=cli_opts.opt_stat)  # passing None if not specified
         elif cmd == "discover":
             # No need to read definition file.
-            machete_client.discover_tree()
+            machete_client.discover_tree(
+                opt_checked_out_since=cli_opts.opt_checked_out_since,
+                opt_list_commits=cli_opts.opt_list_commits,
+                opt_roots=cli_opts.opt_roots,
+                opt_yes=cli_opts.opt_yes)
         elif cmd in {"edit", alias_by_command["edit"]}:
             # No need to read definition file.
             machete_client.edit()
@@ -545,12 +570,18 @@ def launch(orig_args: List[str]) -> None:
             machete_client.read_definition_file()
             branch = get_branch_arg_or_current_branch(cli_opts, git)
             if cli_opts.opt_inferred:
-                print(machete_client.fork_point(branch, use_overrides=False))
+                print(machete_client.fork_point(
+                    branch=branch,
+                    use_overrides=False,
+                    opt_no_detect_squash_merges=cli_opts.opt_no_detect_squash_merges))
             elif cli_opts.opt_override_to:
-                machete_client.set_fork_point_override(branch, cli_opts.opt_override_to)
+                machete_client.set_fork_point_override(branch, AnyRevision.of(cli_opts.opt_override_to))
             elif cli_opts.opt_override_to_inferred:
                 machete_client.set_fork_point_override(
-                    branch, machete_client.fork_point(branch, use_overrides=False))
+                    branch, machete_client.fork_point(
+                        branch=branch,
+                        use_overrides=False,
+                        opt_no_detect_squash_merges=cli_opts.opt_no_detect_squash_merges))
             elif cli_opts.opt_override_to_parent:
                 upstream = machete_client.up_branch.get(branch)
                 if upstream:
@@ -561,13 +592,16 @@ def launch(orig_args: List[str]) -> None:
             elif cli_opts.opt_unset_override:
                 machete_client.unset_fork_point_override(branch)
             else:
-                print(machete_client.fork_point(branch, use_overrides=True))
+                print(machete_client.fork_point(
+                    branch=branch,
+                    use_overrides=True,
+                    opt_no_detect_squash_merges=cli_opts.opt_no_detect_squash_merges))
         elif cmd in {"go", alias_by_command["go"]}:
             machete_client.read_definition_file()
             git.expect_no_operation_in_progress()
             current_branch = git.get_current_branch()
             dest = machete_client.parse_direction(
-                parsed_cli.direction, current_branch, allow_current=False, down_pick_mode=True)
+                parsed_cli.direction, current_branch, allow_current=False, down_pick_mode=True)[0]  # with down_pick_mode=True there is only one element in list allowed
             if dest != current_branch:
                 git.checkout(dest)
         elif cmd == "github":
@@ -595,19 +629,19 @@ def launch(orig_args: List[str]) -> None:
                 machete_client.checkout_github_prs(pr_no)
             elif github_subcommand == "create-pr":
                 current_branch = git.get_current_branch()
-                machete_client.create_github_pr(current_branch, draft=cli_opts.opt_draft)
+                machete_client.create_github_pr(
+                    head=current_branch,
+                    opt_draft=cli_opts.opt_draft,
+                    opt_onto=cli_opts.opt_onto)
             elif github_subcommand == "retarget-pr":
                 current_branch = git.get_current_branch()
                 machete_client.expect_in_managed_branches(current_branch)
                 machete_client.retarget_github_pr(current_branch)
-        elif cmd == "help":
-            # No need to read definition file.
-            print(get_help_description(parsed_cli.topic_or_cmd))
         elif cmd == "is-managed":
             machete_client.read_definition_file()
             branch = get_branch_arg_or_current_branch(cli_opts, git)
             if branch is None or branch not in machete_client.managed_branches:
-                sys.exit(1)
+                exit_script(1)
         elif cmd == "list":
             category = parsed_cli.category
             if category == 'slidable-after' and 'branch' not in parsed_cli_as_dict:
@@ -620,23 +654,23 @@ def launch(orig_args: List[str]) -> None:
             machete_client.read_definition_file()
             res = []
             if category == "addable":
-                def strip_first_fragment(remote_branch: str) -> str:
-                    return re.sub("^[^/]+/", "", remote_branch)
+                def strip_remote_name(remote_branch: RemoteBranchShortName) -> LocalBranchShortName:
+                    return LocalBranchShortName.of(re.sub("^[^/]+/", "", remote_branch))
 
                 remote_counterparts_of_local_branches = utils.map_truthy_only(
-                    lambda _branch: git.get_combined_counterpart_for_fetching_of_branch(_branch),
+                    lambda _branch: LocalBranchShortName.of(git.get_combined_counterpart_for_fetching_of_branch(_branch)),
                     git.get_local_branches())
-                qualifying_remote_branches = excluding(git.get_remote_branches(),
-                                                       remote_counterparts_of_local_branches)
+                qualifying_remote_branches: List[RemoteBranchShortName] = excluding(git.get_remote_branches(),
+                                                                                    {RemoteBranchShortName.of(b) for b in remote_counterparts_of_local_branches})
                 res = excluding(git.get_local_branches(), machete_client.managed_branches) + list(
-                    map(strip_first_fragment, qualifying_remote_branches))
+                    map(strip_remote_name, qualifying_remote_branches))
             elif category == "managed":
                 res = machete_client.managed_branches
             elif category == "slidable":
-                res = machete_client.slidable()
+                res = machete_client.get_slidable_branches()
             elif category == "slidable-after":
                 machete_client.expect_in_managed_branches(parsed_cli.branch)
-                res = machete_client.slidable_after(parsed_cli.branch)
+                res = machete_client.get_slidable_after(parsed_cli.branch)
             elif category == "unmanaged":
                 res = excluding(git.get_local_branches(), machete_client.managed_branches)
             elif category == "with-overridden-fork-point":
@@ -655,42 +689,65 @@ def launch(orig_args: List[str]) -> None:
             machete_client.read_definition_file()
             git.expect_no_operation_in_progress()
             current_branch = git.get_current_branch()
+            if "fork_point" in parsed_cli:
+                machete_client.check_that_forkpoint_is_ancestor_or_equal_to_tip_of_branch(
+                    forkpoint_sha=cli_opts.opt_fork_point, branch=current_branch)
             git.rebase_onto_ancestor_commit(
                 current_branch,
                 cli_opts.opt_fork_point or machete_client.fork_point(
-                    current_branch, use_overrides=True))
+                    branch=current_branch,
+                    use_overrides=True,
+                    opt_no_detect_squash_merges=cli_opts.opt_no_detect_squash_merges),
+                cli_opts.opt_no_interactive_rebase)
         elif cmd == "show":
             direction = parsed_cli.direction
-            if direction == "current" and 'branch' in parsed_cli:
-                raise MacheteException(
-                    '`show current` with a <branch> argument does not make sense')
-            machete_client.read_definition_file(verify_branches=False)
             branch = get_branch_arg_or_current_branch(cli_opts, git)
-            print(
-                machete_client.parse_direction(
-                    direction,
-                    branch,
-                    allow_current=True,
-                    down_pick_mode=False
+            if direction == "current":
+                if 'branch' in parsed_cli:
+                    raise MacheteException(
+                        '`show current` with a <branch> argument does not make sense')
+                print(branch)
+            else:
+                machete_client.read_definition_file(verify_branches=False)
+                print(
+                    '\n'.join(machete_client.parse_direction(
+                        direction,
+                        branch,
+                        allow_current=True,
+                        down_pick_mode=False
+                    ))
                 )
-            )
         elif cmd == "slide-out":
             machete_client.read_definition_file()
             git.expect_no_operation_in_progress()
             branches = parsed_cli_as_dict.get('branches', [git.get_current_branch()])
-            machete_client.slide_out(list(branches))
+            machete_client.slide_out(
+                branches_to_slide_out=list(map(LocalBranchShortName.of, branches)),
+                opt_down_fork_point=cli_opts.opt_down_fork_point,
+                opt_merge=cli_opts.opt_merge,
+                opt_no_interactive_rebase=cli_opts.opt_no_interactive_rebase,
+                opt_no_edit_merge=cli_opts.opt_no_edit_merge)
         elif cmd == "squash":
             machete_client.read_definition_file()
             git.expect_no_operation_in_progress()
             current_branch = git.get_current_branch()
+            if "fork_point" in parsed_cli:
+                machete_client.check_that_forkpoint_is_ancestor_or_equal_to_tip_of_branch(
+                    forkpoint_sha=cli_opts.opt_fork_point, branch=current_branch)
             machete_client.squash(
-                current_branch,
-                cli_opts.opt_fork_point or machete_client.fork_point(
-                    current_branch, use_overrides=True))
+                current_branch=current_branch,
+                opt_fork_point=cli_opts.opt_fork_point or machete_client.fork_point(
+                    branch=current_branch,
+                    use_overrides=True,
+                    opt_no_detect_squash_merges=cli_opts.opt_no_detect_squash_merges))
         elif cmd in {"status", alias_by_command["status"]}:
             machete_client.read_definition_file()
             machete_client.expect_at_least_one_managed_branch()
-            machete_client.status(warn_on_yellow_edges=True)
+            machete_client.status(
+                warn_on_yellow_edges=True,
+                opt_list_commits=cli_opts.opt_list_commits,
+                opt_list_commits_with_hashes=cli_opts.opt_list_commits_with_hashes,
+                opt_no_detect_squash_merges=cli_opts.opt_no_detect_squash_merges)
         elif cmd == "traverse":
             if cli_opts.opt_start_from not in {"here", "root", "first-root"}:
                 raise MacheteException(
@@ -702,25 +759,43 @@ def launch(orig_args: List[str]) -> None:
                     "Valid arguments: `here|nearest-remaining|stay`.")
             machete_client.read_definition_file()
             git.expect_no_operation_in_progress()
-            machete_client.traverse()
+            machete_client.traverse(
+                opt_fetch=cli_opts.opt_fetch,
+                opt_list_commits=cli_opts.opt_list_commits,
+                opt_merge=cli_opts.opt_merge,
+                opt_no_detect_squash_merges=cli_opts.opt_no_detect_squash_merges,
+                opt_no_edit_merge=cli_opts.opt_no_edit_merge,
+                opt_no_interactive_rebase=cli_opts.opt_no_interactive_rebase,
+                opt_push_tracked=cli_opts.opt_push_tracked,
+                opt_push_untracked=cli_opts.opt_push_untracked,
+                opt_return_to=cli_opts.opt_return_to,
+                opt_start_from=cli_opts.opt_start_from,
+                opt_yes=cli_opts.opt_yes)
         elif cmd == "update":
             machete_client.read_definition_file()
             git.expect_no_operation_in_progress()
-            machete_client.update()
+            if "fork_point" in parsed_cli:
+                machete_client.check_that_forkpoint_is_ancestor_or_equal_to_tip_of_branch(
+                    forkpoint_sha=cli_opts.opt_fork_point, branch=git.get_current_branch())
+            machete_client.update(
+                opt_merge=cli_opts.opt_merge,
+                opt_no_edit_merge=cli_opts.opt_no_edit_merge,
+                opt_no_interactive_rebase=cli_opts.opt_no_interactive_rebase,
+                opt_fork_point=cli_opts.opt_fork_point)
         elif cmd == "version":
             version()
-            sys.exit()
+            exit_script()
 
     except (argparse.ArgumentError, argparse.ArgumentTypeError) as e:
         print(get_short_general_usage())
         sys.stderr.write(f"\n{e}\n")
-        sys.exit(2)
+        exit_script(2)
     except MacheteException as e:
         sys.stderr.write(f"\n{e}\n")
-        sys.exit(1)
+        exit_script(1)
     except KeyboardInterrupt:
         sys.stderr.write("\nInterrupted by the user")
-        sys.exit(1)
+        exit_script(1)
     except StopInteraction:
         pass
     finally:
@@ -735,7 +810,6 @@ def launch(orig_args: List[str]) -> None:
 
 
 def main() -> None:
-    validate_python_version()
     launch(sys.argv[1:])
 
 
