@@ -1,3 +1,4 @@
+from enum import IntEnum
 import os
 
 MAX_COUNT_FOR_INITIAL_LOG = 10
@@ -21,14 +22,15 @@ class EscapeCodes:
     RED = '\033[91m'
 
 
-# TODO (#117): extract to namespace, use mypy
-NO_REMOTES = 0
-UNTRACKED = 1
-IN_SYNC_WITH_REMOTE = 2
-BEHIND_REMOTE = 3
-AHEAD_OF_REMOTE = 4
-DIVERGED_FROM_AND_OLDER_THAN_REMOTE = 5
-DIVERGED_FROM_AND_NEWER_THAN_REMOTE = 6
+class SyncToRemoteStatuses(IntEnum):
+    NO_REMOTES = 0
+    UNTRACKED = 1
+    IN_SYNC_WITH_REMOTE = 2
+    BEHIND_REMOTE = 3
+    AHEAD_OF_REMOTE = 4
+    DIVERGED_FROM_AND_OLDER_THAN_REMOTE = 5
+    DIVERGED_FROM_AND_NEWER_THAN_REMOTE = 6
+
 
 GIT_FORMAT_PATTERNS = {
     "author name": "%aN",
