@@ -24,7 +24,7 @@ short_docs: Dict[str, str] = {
     "slide-out": "Slide out the current branch and sync its downstream (child) branches with its upstream (parent) branch via rebase or merge",
     "squash": "Squash the unique history of the current branch into a single commit",
     "status": "Display formatted tree of branch dependencies, including info on their sync with upstream branch and with remote",
-    "traverse": "Walk through the tree of branch dependencies and rebase, merge, slide out, push and/or pull each branch one by one",
+    "traverse": "Walk through the tree of branch dependencies and rebase, merge, slide out, push and/or pull each branch one by one. By default starts from current branch",
     "update": "Sync the current branch with its upstream (parent) branch via rebase or merge",
     "version": "Display the version and exit"
 }
@@ -305,8 +305,9 @@ long_docs: Dict[str, str] = {
         To allow GitHub API access for private repositories (and also to perform side-effecting actions like opening a PR, even in case of public repositories),
         a GitHub API token with `repo` scope is required, see `https://github.com/settings/tokens`. This will be resolved from the first of:
         1. `GITHUB_TOKEN` env var,
-        2. current auth token from the `gh` GitHub CLI,
-        3. current auth token from the `hub` GitHub CLI.
+        2. content of the .github-token file in the home directory (`~`),
+        3. current auth token from the `gh` GitHub CLI,
+        4. current auth token from the `hub` GitHub CLI.
 
         <b>`anno-prs`:</b>
 
@@ -382,7 +383,7 @@ long_docs: Dict[str, str] = {
             * exactly once during a `slide-out` execution (even if multiple branches are slid out),
             * zero or more times during `traverse` (every time a slide-out operation is confirmed).
 
-            If the hook returns a non-zero exit code, then the execution of the command is aborted,
+            If the hook returns a non-zero exit code, then an error is raised and the execution of the command is aborted,
             i.e. `slide-out` won't attempt rebase of the new downstream branches and `traverse` won't continue the traversal.
             In case of `advance` there is no difference (other than exit code of the entire `advance` command being non-zero),
             since slide-out is the last operation that happens within `advance`.
@@ -397,7 +398,7 @@ long_docs: Dict[str, str] = {
             1. what is going to be the new base for the rebased commits,
             2. what is the fork point - the place where the rebased history diverges from the upstream history,
             3. what branch is rebased.
-            If the hook returns a non-zero exit code, the entire rebase is aborted.
+            If the hook returns a non-zero exit code, an error is raised and the entire rebase is aborted.
 
             Note: this hook is independent from git's standard `pre-rebase` hook.
             If machete-pre-rebase returns zero, the execution flow continues to `git rebase`, which may also run `pre-rebase` hook if present.
@@ -610,7 +611,9 @@ long_docs: Dict[str, str] = {
                                        [--[no-]push] [--[no-]push-untracked]
                                        [--return-to=WHERE] [--start-from=WHERE] [-w|--whole] [-W] [-y|--yes]</b>
 
-        Traverses the branch dependency tree in pre-order (i.e. simply in the order as they occur in the definition file) and for each branch:
+        Traverses the branch dependency tree in pre-order (i.e. simply in the order as they occur in the definition file).
+        By default traverse starts from current branch, however this behaviour can be customized using options: `--starts-from=`, `--whole` or `-w`, `-W`.
+        For each branch, the command:
         * detects if the branch is merged to its parent/upstream
           - by commit equivalency (default), or by strict detection of merge commits (if `--no-detect-squash-merges` passed),
           - if so, asks the user whether to slide out the branch from the dependency tree (typically branches are longer needed after they're merged);
