@@ -7,13 +7,13 @@ import re
 import shutil
 import subprocess
 from pathlib import Path
-from typing import Dict, List, Optional, Any, Tuple
+from typing import Any, Dict, List, Optional, Tuple
 import urllib.request
 from urllib.error import HTTPError
 
 from git_machete.utils import debug, fmt, find_or_none
 from git_machete.exceptions import MacheteException, UnprocessableEntityHTTPError
-from git_machete.git_operations import GitContext
+from git_machete.git_operations import GitContext, LocalBranchShortName
 
 
 GITHUB_TOKEN_ENV_VAR = 'GITHUB_TOKEN'
@@ -109,7 +109,20 @@ def __get_github_token() -> Optional[str]:
     def get_token_from_env() -> Optional[str]:
         return os.environ.get(GITHUB_TOKEN_ENV_VAR)
 
-    return get_token_from_env() or get_token_from_gh() or get_token_from_hub()
+    def get_token_from_file_in_home_directory() -> Optional[str]:
+        required_file_name = '.github-token'
+        file_full_path = os.path.expanduser(f'~/{required_file_name}')
+
+        if os.path.isfile(file_full_path):
+            with open(file_full_path) as file:
+                return file.read().strip()
+
+        return None
+
+    return (get_token_from_env() or
+            get_token_from_file_in_home_directory() or
+            get_token_from_gh() or
+            get_token_from_hub())
 
 
 def __fire_github_api_request(method: str, path: str, token: Optional[str], request_body: Optional[Dict[str, Any]] = None) -> Any:
@@ -134,7 +147,9 @@ def __fire_github_api_request(method: str, path: str, token: Optional[str], requ
             return parsed_response_body
     except HTTPError as err:
         if err.code == http.HTTPStatus.UNPROCESSABLE_ENTITY:
-            raise UnprocessableEntityHTTPError(str(err.reason))
+            error_response = json.loads(err.read().decode())
+            error_reason: str = error_response['errors'][0]['message']
+            raise UnprocessableEntityHTTPError(error_reason)
         elif err.code in (http.HTTPStatus.UNAUTHORIZED, http.HTTPStatus.FORBIDDEN):
             first_line = f'GitHub API returned {err.code} HTTP status with error message: `{err.reason}`\n'
             if token:
@@ -147,7 +162,8 @@ def __fire_github_api_request(method: str, path: str, token: Optional[str], requ
                                  'Visit `https://github.com/settings/tokens` to generate a new one.')
         elif err.code == http.HTTPStatus.NOT_FOUND:
             raise MacheteException(
-                f'Given endpoint: {url}, not found in Github')  # TODO (#164): make dedicated exception here
+                f'Given endpoint: {url}, not found in Github or you don\'t have'
+                f' the permission to access it (expired token?).')  # TODO (#164): make dedicated exception here
         else:
             first_line = fmt(f'GitHub API returned {err.code} HTTP status with error message: `{err.reason}`\n')
             raise MacheteException(first_line + "Please open an issue regarding this topic under link: https://github.com/VirtusLab/git-machete/issues/new")
@@ -190,7 +206,7 @@ def add_reviewers_to_pull_request(org: str, repo: str, number: int, reviewers: L
     __fire_github_api_request('POST', f'/repos/{org}/{repo}/pulls/{number}/requested_reviewers', token, request_body)
 
 
-def set_base_of_pull_request(org: str, repo: str, number: int, base: str) -> None:
+def set_base_of_pull_request(org: str, repo: str, number: int, base: LocalBranchShortName) -> None:
     token: Optional[str] = __get_github_token()
     request_body: Dict[str, str] = {'base': base}
     __fire_github_api_request('PATCH', f'/repos/{org}/{repo}/pulls/{number}', token, request_body)
@@ -203,7 +219,7 @@ def set_milestone_of_pull_request(org: str, repo: str, number: int, milestone: s
     __fire_github_api_request('PATCH', f'/repos/{org}/{repo}/issues/{number}', token, request_body)
 
 
-def derive_pull_request_by_head(org: str, repo: str, head: str) -> Optional[GitHubPullRequest]:
+def derive_pull_request_by_head(org: str, repo: str, head: LocalBranchShortName) -> Optional[GitHubPullRequest]:
     token: Optional[str] = __get_github_token()
     prs = __fire_github_api_request('GET', f'/repos/{org}/{repo}/pulls?head={org}:{head}', token)
     if len(prs) >= 1:
@@ -248,6 +264,6 @@ def get_pull_request_by_number_or_none(number: int, org: str, repo: str) -> Opti
         return None
 
 
-def checkout_pr_refs(git: GitContext, remote: str, pr_number: int, branch: str) -> None:
+def checkout_pr_refs(git: GitContext, remote: str, pr_number: int, branch: LocalBranchShortName) -> None:
     git.fetch_ref(remote, f'pull/{pr_number}/head:{branch}')
     git.checkout(branch)
